function [vs,cumvar,xs,vso] = covsel(x,y,lv);
  
  // [num,crbvar] = covsel(x,y,lv)
  // Selection of variable from covariance maximisation
  //
  // Input arguments
  // ===============
  // x:      matrix n x p
  // y:      matrix n x q
  // lv:     desired number of variables
  //
  // Output arguments
  // ================
  // num    : number (rank) of selected variables
  // crbvar : evolution of cumulated variance of X (first column) and Y (second one)
  //
  // This function is called by 'covariance_select' (SAISIR format)
  // Written by JM Roger, CEMAGREF, Montpellier
  //
  // see also: covariance_select, selectvar
  // last update: 2012, march, 30
  
  xs=div(x);
  ys=div(y);

  [nx,px] = size(xs.d);
  [ny,py] = size(ys.d);

  // default value for lv = all the variables of x
  if  argn(2)==2 then lv=px;
  end
  
  crbvar=zeros(lv,2);  
  
  if nx ~= ny
  error('Unconsistent number of rows')
  end

  lv = min( lv, rank(xs.d));

  z=covsel_mncn(xs.d);
  t=covsel_auto(ys.d);

  vartot=[trace(z'*z) trace(t'*t)];

  for i=1:lv
      [cm,j] = max( diag( z'*t*t'*z ),'r');
      v = z(:,j);
      z = ( eye(nx,nx)-(v*v')/(v'*v) )*z;
      crbvar(i,1) = (vartot(1)-trace(z'*z))/vartot(1);
      t = ( eye(nx,nx)-(v*v')/(v'*v) )*t;
      crbvar(i,2) = (vartot(2)-trace(t'*t))/vartot(2);
      num(i)=j;
  end;

  // extraction of the selected variables from x
  num2=gsort(num,'r','i');
  xsel.d=xs.d(:,num2);
  xsel.i=xs.i;
  xsel.v=xs.v(num2);


  vs=num;
  cumvar=crbvar;
  xs=xsel;
  vso=num2;
  
  //Div
  xs=div(xs);
  


endfunction


// Local function covsel_auto =====================================

function [ax,mx,stdx] = covsel_auto(x)
  //  Autoscales a matrix (x) and returns the resulting matrix (ax)
  //  with mean-zero unit variance columns, a vector of means (mx)
  //  and a vector of standard deviations (stdx) used in the scaling.
  [m,n] = size(x);
  mx    = mean(x,1);
  stdx  = st_deviation(x,1);
  ax    = (x-mx(ones(m,1),:))./stdx(ones(m,1),:);
endfunction


// local function covsel_mncn=====================

function [mcx,mx] = covsel_mncn(x)
  //  Mean centers matrix (x), returning a matrix with
  //  mean zero columns (mcx) and the vector of means
  //  (mx) used in the scaling.
  [m,n] = size(x);
  mx    = mean(x,1);
  mcx   = (x-mx(ones(m,1),:));
endfunction





